import subprocess
from iupdatable.logging import Logger


class CSProduct(object):

    @staticmethod
    def get():
        try:
            completed = subprocess.run('wmic csproduct get /value', stdout=subprocess.PIPE, universal_newlines=True)
            output = str(completed.stdout).strip().replace('\n\n', '\n')
            items = output.split("\n")
            result = {}
            for item in items:
                key_value = item.split("=")
                result[key_value[0]] = key_value[1]
            return result
        except Exception as e:
            Logger.error(repr(e), is_with_debug_info=True)
            return None

    @staticmethod
    def get_caption():
        """
        获取 Caption 的值
        :return: 成功：返回 Caption 的值，如：Computer System Product； 失败：返回 None
        """
        cs_product = CSProduct.get()
        if cs_product:
            return cs_product["Caption"]
        else:
            return None

    @staticmethod
    def get_description():
        """
        获取 Description 的值
        :return: 成功：返回 Description 的值，如：Computer System Product； 失败：返回 None
        """
        cs_product = CSProduct.get()
        if cs_product:
            return cs_product["Description"]
        else:
            return None

    @staticmethod
    def get_identifying_number():
        """
        获取 IdentifyingNumber 的值
        :return: 成功：返回 IdentifyingNumber 的值，如：PDVC400012152042979202； 失败：返回 None
        """
        cs_product = CSProduct.get()
        if cs_product:
            return cs_product["IdentifyingNumber"]
        else:
            return None

    @staticmethod
    def get_name():
        """
        获取 Name 的值
        :return: 成功：返回 Name 的值，如：Veriton M4610G； 失败：返回 None
        """
        cs_product = CSProduct.get()
        if cs_product:
            return cs_product["Name"]
        else:
            return None

    @staticmethod
    def get_sku_number():
        """
        获取 SKUNumber 的值
        :return: 成功：返回 SKUNumber 的值，可能为空字符串； 失败：返回 None
        """
        cs_product = CSProduct.get()
        if cs_product:
            return cs_product["SKUNumber"]
        else:
            return None

    @staticmethod
    def get_uuid():
        """
        获取 UUID 的值
        :return: 成功：返回 UUID 的值，如：A2DC9CC8-30A8-1120-1228-222416000000； 失败：返回 None
        """
        cs_product = CSProduct.get()
        if cs_product:
            return cs_product["UUID"]
        else:
            return None

    @staticmethod
    def get_vendor():
        """
        获取 Vendor 的值
        :return: 成功：返回 Vendor 的值，如：Acer； 失败：返回 None
        """
        cs_product = CSProduct.get()
        if cs_product:
            return cs_product["Vendor"]
        else:
            return None

    @staticmethod
    def get_version():
        """
        获取 Version 的值
        :return: 成功：返回 Version 的值，默认可能为空； 失败：返回 None
        """
        cs_product = CSProduct.get()
        if cs_product:
            return cs_product["Version"]
        else:
            return None



import os
import csv
import pathlib


class File(object):

    @staticmethod
    def read(file_path: str, encoding="utf_8_sig") -> str:
        """"
        读取文件
        :param file_path: 完整的文件路径
        :param encoding: 编码格式，默认utf_8_sig，这是Windows下记事本utf8的编码格式
        :return:
        """
        with open(file_path, encoding=encoding) as f:  # 默认为utf8编码
            return f.read()

    @staticmethod
    def write(file_path: str, content: str, encoding="utf_8_sig") -> None:
        """"
        写入文件
        :param file_path: 完整的文件路径
        :param content: 待写入内容
        :param encoding: 编码格式，默认utf_8_sig，这是Windows下记事本utf8的编码格式
        :return:
        """
        with open(file_path, "w", encoding=encoding) as f:
            f.write(content)

    @staticmethod
    def append(file_path: str, content: str, encoding="utf_8_sig") -> None:
        """"
        追加写入文件
        :param file_path: 完整的文件路径
        :param content: 待写入内容
        :param encoding: 编码格式，默认utf_8_sig，这是Windows下记事本utf8的编码格式
        :return:
        """
        with open(file_path, "a", encoding=encoding) as f:
            f.write(content)

    @staticmethod
    def append_new_line(file_path: str, content: str, encoding="utf_8_sig") -> None:
        """"
        新建一行，然后追加写入文件
        新文件或空白文件，则不添加新行
        :param file_path: 完整的文件路径
        :param content: 待写入内容
        :param encoding: 编码格式，默认utf_8_sig，这是Windows下记事本utf8的编码格式
        :return:
        """
        is_exist = os.path.exists(file_path)

        if is_exist:
            with open(file_path, "a", encoding=encoding) as file:
                size = os.path.getsize(file_path)
                if size == 3 and encoding.lower() == "utf_8_sig":  # utf_8_sig的txt文件会在开头添加三个字节的标识
                    file.write(content)
                elif size == 0 and encoding.lower() != "utf_8_sig":
                    file.write(content)
                else:
                    file.write("\n" + content)
        else:
            with open(file_path, "a", encoding=encoding) as new_file:
                new_file.write(content)

    @staticmethod
    def read_lines(file_path: str, remove_empty_line=True, remove_empty_space=True, encoding="utf_8_sig") -> []:
        """
        按行一次性读取文件
        :param file_path: 完整的文件路径
        :param remove_empty_line: 是否去除空白行
        :param remove_empty_space: 是否移除每行前后的空白
        :param encoding: 编码格式，默认utf_8_sig，这是Windows下记事本utf8的编码格式
        :return:
        """
        lines = []
        with open(file_path, encoding=encoding) as f:  # 默认为utf8编码
            for line in f.readlines():
                if remove_empty_space:
                    line = line.strip()
                if remove_empty_line:
                    line = line.rstrip('\n')  # 去掉每行的回车
                    if line:  # 去掉空行
                        lines.append(line)
                else:
                    line = line.rstrip('\n')  # 去掉每行的回车
                    lines.append(line)
        return lines

    @staticmethod
    def write_lines(file_path: str, lines: [], encoding="utf_8_sig") -> None:
        """
        按行一次性写入文件
        :param file_path: 完整的文件路径
        :param lines: 待写入的多行内容
        :param encoding: 编码格式，默认utf_8_sig，这是Windows下记事本utf8的编码格式
        :return:
        """
        with open(file_path, "w", encoding=encoding) as f:
            str_lines = [str(line) for line in lines]  # 针对数组中有数字的情况
            f.write('\n'.join(str_lines))

    @staticmethod
    def write_csv(file_path: str, rows: []) -> None:
        """
        写入CSV文件
        :param file_path: 完整的文件路径，如："D:\\12.csv","1.csv"
        :param rows: 待写入的多行数据，如：[['row11','row12','row13'],['row21','row22','row23']]
        :return:
        """
        with open(file_path, 'w', encoding='utf_8_sig', newline='') as f:
            csv_write = csv.writer(f)
            for oneRow in rows:
                csv_write.writerow(oneRow)

    @staticmethod
    def read_csv(file_path: str, encoding="utf_8_sig") -> []:
        """
        读取CSV文件
        :param file_path: 完整的文件路径，如："D:\\12.csv","1.csv"
        :param encoding: 文件编码，默认为utf_8_sig
        :return:
        """
        rows = []
        with open(file_path, 'r', encoding=encoding) as f:
            csv_reader = csv.reader(f)
            for oneRow in csv_reader:
                rows.append(oneRow)
        return rows

    @staticmethod
    def exist_within_extensions(file_path: str, extension_list: []) -> bool:
        """
        检查一个文件是否存在（在指定的几种格式中）
        :param file_path: 文件的路径
        :param extension_list: 关心的格式,如：["jpg", "png", "bmp"]
        :return: 有任意一种格式的存在即认为文件存在
        """
        if os.path.isfile(file_path):
            return True
        win_path = pathlib.PureWindowsPath(file_path)
        file_path_no_extension = str(win_path.parent) + "\\" + win_path.stem
        for one_extension in extension_list:
            new_file_path = file_path_no_extension + "." + one_extension
            if os.path.isfile(new_file_path):
                return True
        return False

    @staticmethod
    def get_file_path_within_extensions(file_path: str, extension_list: []) -> str:
        """
        获取一个文件的路径（在指定的几种格式中）
        适用场景：
        想要获取一个指定路径下的文件，如：c:\testfile，
        但并不关心其具体是什么格式，只要是在extension_list指定的其中一种即可（靠前的优先）
        :param file_path: 文件的路径，如：c:\testfile，一个没有拓展名的文件（拓展名可有可无）
        :param extension_list: 关心的格式,如：["jpg", "png", "bmp"]
        :return: 不存在则返回None
        """
        if os.path.isfile(file_path):
            return file_path
        win_path = pathlib.PureWindowsPath(file_path)
        file_path_no_extension = str(win_path.parent) + "\\" + win_path.stem
        for one_extension in extension_list:
            new_file_path = file_path_no_extension + "." + one_extension
            if os.path.isfile(new_file_path):
                return new_file_path
        return None



import os
import sys
import inspect


class Directory(object):

    @staticmethod
    def get_current_directory() -> str:
        """"
        获取程序的当前路径
        :return:
        成功：当前程序的完整路径，如：d:\test
        失败：返回空字符串
        """
        try:
            # determine if application is a script file or frozen exe
            if getattr(sys, 'frozen', False):
                application_path = os.path.dirname(sys.executable)
            else:
                frame = inspect.stack()[1]
                module = inspect.getmodule(frame[0])
                filename = module.__file__
                application_path = os.path.dirname(filename)
            return application_path
        except Exception as e:
            print('get_current_directory Exception:')
            print(repr(e))
            return ""

    @staticmethod
    def get_files(directory_name: str, is_include_dir=False) -> []:
        """
        返回指定目录中的文件名（包括它们的路径）,是否包含子目录可选。
        :param directory_name: 完整的文件夹路径，如：d:\test
        :param is_include_dir: 是否包含子目录，默认不包含
        :return: 成功：返回文件名集合，失败：返回空的数组
        """
        try:
            full_path_list = []
            file_names = os.listdir(directory_name)
            file_names.sort()
            for file_name in file_names:
                full_path = os.path.join(directory_name, file_name)
                if os.path.isdir(full_path):
                    if is_include_dir:
                        full_path_list.append(full_path)
                else:
                    full_path_list.append(full_path)
            return full_path_list
        except Exception as e:
            print('get_files Exception:')
            print(repr(e))
            return []



import os
import requests
import time
import uuid
from iupdatable.system.hardware.CSProduct import CSProduct
import ctypes
import locale
import urllib


class UMeng(object):

    @classmethod
    def log_stat(cls, id, page, category='', action='', label='', value=0):
        os.environ['NO_PROXY'] = 'cnzz.com'
        host_url = 'https://ei.cnzz.com/stat.htm'
        um_uuid = cls.get_um_uuid()
        user32 = ctypes.windll.user32
        resolution = user32.GetSystemMetrics(0), user32.GetSystemMetrics(1)
        lang = locale.getdefaultlocale()[0]
        ei = urllib.parse.quote_plus(f'{category}|{action}|{label}|{value}|')
        rnd = int(time.time())
        page = urllib.parse.quote_plus('http://' + page)
        url = host_url + f'?id={id}&lg={lang}&ei={ei}&p={page}' \
                         f'&umuuid={um_uuid}&showp={resolution[0]}x{resolution[1]}&h=1'
        response = requests.get(url)

    @staticmethod
    def get_um_uuid():
        # section1
        timestamp = int(round(time.time() * 1000))
        d = 0
        while timestamp == int(round(time.time() * 1000)):
            d += 1
        section1 = format(timestamp, 'x') + format(d, 'x')
        # section2
        section2 = str(uuid.uuid1()).replace('-', '')[:14]
        # section3
        section3 = CSProduct.get_uuid().replace('-', '')[:6]
        # section4
        user32 = ctypes.windll.user32
        screensize = user32.GetSystemMetrics(0), user32.GetSystemMetrics(1)
        section4 = format(screensize[0] * screensize[1], 'x')
        return '{0}-{1}-{2}-{3}'.format(section1, section2, section3, section4)



import socket
from urllib.parse import urlparse


class HttpWebRequest(object):

    @classmethod
    def get(cls, url: str, headers=None):
        return cls._request("GET", url, headers)

    @classmethod
    def _request(cls, method: str, url: str, headers: dict):
        try:
            headers_str = ""
            if method == 'GET':
                headers_str += "GET "
            elif method == "POST":
                headers_str += "POST "
            else:
                return None

            parsed_uri = urlparse(url)
            headers_str += parsed_uri.path + "?" + parsed_uri.query + " HTTP/1.1\r\n"
            host = parsed_uri.netloc
            scheme = parsed_uri.scheme  # 'http'、'https'、''
            headers_str += "Host: " + host + "\r\n"

            if headers:
                header_keys_lower = [k.lower() for k, v in headers.items()]
            else:
                headers_str += "Accept-Encoding: gzip, deflate\r\n"
                headers_str += "Accept: */*\r\n"
                headers_str += "Connection: keep-alive\r\n\r\n"

            tcp_client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            tcp_client.settimeout(200)  # 200秒
            if scheme == 'https':
                port = 443
            else:
                port = 80
            tcp_client.connect((host, port))
            tcp_client.sendall(str.encode(headers_str))
            result = bytearray(b'')
            while True:
                data = tcp_client.recv(1024)
                result.extend(data)
                if not data:
                    break
            # TODO
            # 应返回：消息头 + 内容， 需要结构化数据，同时数部分应该解码
            # 参考：http://www.zeroplace.cn/article.asp?id=985
            return result
        except Exception as e:
            print(repr(e))
        finally:
            if tcp_client:
                tcp_client.close()



import threading
from .LogLevel import LogLevel
from iupdatable.system.io.Directory import Directory
import logging as sys_logging
import logging.handlers
import datetime
import sys
from colorlog import ColoredFormatter


class Logger(object):
    _instance_lock = threading.Lock()

    def __init__(self):
        pass

    def __new__(cls, *args, **kwargs):
        if not hasattr(Logger, "_instance"):
            with Logger._instance_lock:
                if not hasattr(Logger, "_instance"):
                    Logger._instance = object.__new__(cls)
                    cls._is_init = False
        return Logger._instance

    @classmethod
    def get_instance(cls):
        """
        获取日志实例
        :return:
        """
        return Logger()

    @classmethod
    def config(cls,
               log_level=LogLevel.DEBUG,
               log_file_full_path="all.log",
               date_format="%Y-%m-%d %H:%M:%S",
               is_with_datetime=True,
               is_with_level=True,
               is_output_file=True,
               is_output_console=True,
               is_colored=True
               ):
        """
        配置日志
        :param log_level:           日志输出等级，参考：LogLevel的值
        :param log_file_full_path:  日志输出的文件名的完整路径，默认：all.log
        :param date_format:         每天日志前日期时间的格式，默认: %Y-%m-%d %H:%M:%S
        :param is_with_datetime:    是否在每条日志前添加日期时间
        :param is_with_level:       是否在每天日志前添加日志等级
        :param is_output_file:      是否将日志输出到文件
        :param is_output_console:   是否将日志输出到控制台
        :param is_colored:          是否将控制台的输出设为彩色
        :return:
        """
        print("log file: " + log_file_full_path)
        cls._logger = cls._get_config_logger("my_logger", is_with_src_info=False, log_level=log_level,
                                             log_file_full_path=log_file_full_path, date_format=date_format,
                                             is_with_datetime=is_with_datetime, is_with_level=is_with_level,
                                             is_output_file=is_output_file, is_output_console=is_output_console,
                                             is_colored=is_colored)
        # 输出调试信息，如：文件路径，代码行定位
        cls._debug_logger = cls._get_config_logger("my_debug_logger", is_with_src_info=True, log_level=log_level,
                                                   log_file_full_path=log_file_full_path, date_format=date_format,
                                                   is_with_datetime=is_with_datetime, is_with_level=is_with_level,
                                                   is_output_file=is_output_file, is_output_console=is_output_console,
                                                   is_colored=is_colored)
        cls._is_init = True

    @classmethod
    def _get_config_logger(cls,
                           logger_name,
                           log_level=LogLevel.DEBUG,
                           log_file_full_path="all.log",
                           date_format="%Y-%m-%d %H:%M:%S",
                           is_with_datetime=True,
                           is_with_level=True,
                           is_with_src_info=False,
                           is_output_file=True,
                           is_output_console=True,
                           is_colored=True
                           ):
        """
        获取一个配置后的logger
        :param logger_name:         日志对象的名称，用户指定
        :param log_level:           日志输出等级，参考：LogLevel的值
        :param log_file_full_path:  日志输出的文件名的完整路径，默认：all.log
        :param date_format:         每天日志前日期时间的格式，默认: %Y-%m-%d %H:%M:%S
        :param is_with_datetime:    是否在每条日志前添加日期时间
        :param is_with_level:       是否在每天日志前添加日志等级
        :param is_with_src_info:    是否包含源码定位信息，如:所在文件及代码行
        :param is_output_file:      是否将日志输出到文件
        :param is_output_console:   是否将日志输出到控制台
        :param is_colored:          是否将控制台的输出设为彩色
        :return: 返回一个配置完成的logger
        """
        if log_file_full_path == "all.log" or len(log_file_full_path.strip()) == 0:
            log_file_full_path = Directory.get_current_directory() + "/all.log"

        logger = sys_logging.getLogger(logger_name)
        logger.handlers.clear()  # 清空handlers，防止是二次调用config

        # 设置日志输出格式
        formatter_prefix = ""
        if is_with_level:
            formatter_prefix += "[%(levelname)-8s]\t"  # -8s表示总计占8个字符，负号表示左对齐
        if is_with_datetime:
            formatter_prefix += "%(asctime)s\t>>\t"
        if is_with_src_info:
            formatter_prefix += "%(myfullpath)s, Function: %(myfuncName)s, Line: %(mylineno)s:\n\t\t\t\t\t\t\t\t\t"
        formatter_str = formatter_prefix + "%(message)s"
        cls._Level = log_level
        logger.setLevel(log_level)
        logger_formatter = sys_logging.Formatter(fmt=formatter_str, datefmt=date_format)

        # 设置日志输出到文件
        if is_output_file:
            # 输出到文件的 handler
            logger_file_handler = sys_logging.handlers.TimedRotatingFileHandler(
                log_file_full_path,
                when='midnight', interval=1, backupCount=7,
                atTime=datetime.time(0, 0, 0, 0))
            logger_file_handler.suffix = "%Y-%m-%d.log"
            logger_file_handler.setFormatter(logger_formatter)
            logger.addHandler(logger_file_handler)

        # 设置日志色彩相关
        # 色彩只和控制台相关，故本段代码要放在文件handler之后，控制台handler之前
        if is_colored:
            formatter_str = "%(log_color)s" + formatter_str + "%(reset)s"
            logger_formatter = ColoredFormatter(fmt=formatter_str, datefmt=date_format)

        # 设置日志输出到控制台
        if is_output_console:
            logger_console_handler = sys_logging.StreamHandler(stream=sys.stdout)  # 输出到控制台的 handler
            logger_console_handler.setFormatter(logger_formatter)
            logger.addHandler(logger_console_handler)
        return logger

    @classmethod
    def debug(cls, message, is_with_debug_info=False):
        if not cls._is_init:
            cls.config()
        if is_with_debug_info:
            # 获取调用者的信息（该部分代码不能再次封装成函数）
            full_path = sys._getframe(1).f_code.co_filename
            func_name = sys._getframe(1).f_code.co_name
            line_no = sys._getframe(1).f_lineno
            cls._extra_args = {'myfullpath': full_path, 'myfuncName': func_name, 'mylineno': line_no}
            cls._debug_logger.debug(message, extra=cls._extra_args)
        else:
            cls._logger.debug(message)

    @classmethod
    def info(cls, message, is_with_debug_info=False):
        if not cls._is_init:
            cls.config()
        if is_with_debug_info:
            # 获取调用者的信息（该部分代码不能再次封装成函数）
            full_path = sys._getframe(1).f_code.co_filename
            func_name = sys._getframe(1).f_code.co_name
            line_no = sys._getframe(1).f_lineno
            cls._extra_args = {'myfullpath': full_path, 'myfuncName': func_name, 'mylineno': line_no}
            cls._debug_logger.info(message, extra=cls._extra_args)
        else:
            cls._logger.info(message)

    @classmethod
    def warning(cls, message, is_with_debug_info=False):
        if not cls._is_init:
            cls.config()
        if is_with_debug_info:
            # 获取调用者的信息（该部分代码不能再次封装成函数）
            full_path = sys._getframe(1).f_code.co_filename
            func_name = sys._getframe(1).f_code.co_name
            line_no = sys._getframe(1).f_lineno
            cls._extra_args = {'myfullpath': full_path, 'myfuncName': func_name, 'mylineno': line_no}
            cls._debug_logger.warning(message, extra=cls._extra_args)
        else:
            cls._logger.warning(message)

    @classmethod
    def error(cls, message, is_with_debug_info=False):
        if not cls._is_init:
            cls.config()
        if is_with_debug_info:
            # 获取调用者的信息（该部分代码不能再次封装成函数）
            full_path = sys._getframe(1).f_code.co_filename
            func_name = sys._getframe(1).f_code.co_name
            line_no = sys._getframe(1).f_lineno
            cls._extra_args = {'myfullpath': full_path, 'myfuncName': func_name, 'mylineno': line_no}
            cls._debug_logger.error(message, extra=cls._extra_args)
        else:
            cls._logger.error(message)

    @classmethod
    def critical(cls, message, is_with_debug_info=False):
        if not cls._is_init:
            cls.config()
        if is_with_debug_info:
            # 获取调用者的信息（该部分代码不能再次封装成函数）
            full_path = sys._getframe(1).f_code.co_filename
            func_name = sys._getframe(1).f_code.co_name
            line_no = sys._getframe(1).f_lineno
            cls._extra_args = {'myfullpath': full_path, 'myfuncName': func_name, 'mylineno': line_no}
            cls._debug_logger.critical(message, extra=cls._extra_args)
        else:
            cls._logger.critical(message)

    @classmethod
    def fatal(cls, message, is_with_debug_info=False):
        if not cls._is_init:
            cls.config()
        if is_with_debug_info:
            # 获取调用者的信息（该部分代码不能再次封装成函数）
            full_path = sys._getframe(1).f_code.co_filename
            func_name = sys._getframe(1).f_code.co_name
            line_no = sys._getframe(1).f_lineno
            cls._extra_args = {'myfullpath': full_path, 'myfuncName': func_name, 'mylineno': line_no}
            cls._debug_logger.critical(message, extra=cls._extra_args)
        else:
            cls._logger.critical(message)



import Lib.base64 as sys_base64


class Base64(object):

    @staticmethod
    def encode(src: str, encoding="utf8") -> str:
        """
        base64编码
        :param src: 待编码文本
        :param encoding: 编码类型
        :return:
        """
        return sys_base64.b64encode((bytes(src, encoding=encoding))).decode()

    @staticmethod
    def decode(src: str, encoding="utf8") -> str:
        """
        base64解码
        :param src: 待解码文本
        :param encoding: 编码类型
        :return:
        """
        return sys_base64.b64decode(bytes(src, encoding=encoding)).decode()

    @staticmethod
    def encode_multilines(src_lines: [], encoding="utf8") -> []:
        """
        base64多行编码
        :param src_lines: 待编码的多行文本
        :param encoding: 编码类型
        :return:
        """
        base64_lines = []
        for line in src_lines:
            base64_str = sys_base64.b64encode(bytes(line, encoding=encoding)).decode()  # 编码
            base64_lines.append(base64_str)
        return base64_lines

    @staticmethod
    def decode_multilines(src_lines: [], encoding="utf8") -> []:
        """
        base64多行解码
        :param src_lines: 待解码的多行文本
        :param encoding: 编码类型
        :return:
        """
        base64_lines = []
        for line in src_lines:
            base64_str = sys_base64.b64decode(bytes(line, encoding=encoding)).decode()  # 解码
            base64_lines.append(base64_str)
        return base64_lines
